@@ -6,6 +6,7 @@ from collections import namedtuple
 import resource
 import logging
 import numpy as np
+from time import perf_counter
 
 try:
     import cupy as cp
@@ -19,14 +20,11 @@ from kernel_tuner.c import CFunctions
 from kernel_tuner.nvml import NVMLObserver
 import kernel_tuner.util as util
 
-
 try:
     import torch
 except ImportError:
     torch = util.TorchPlaceHolder()
 
-
-
 _KernelInstance = namedtuple("_KernelInstance", ["name", "kernel_source", "kernel_string", "temp_files", "threads", "grid", "params", "arguments"])
 
 
@@ -86,7 +84,6 @@ class KernelSource(object):
         :returns: A string containing the kernel code.
         :rtype: string
         """
-        #logging.debug('get_kernel_string called with %s', str(kernel_source))
         logging.debug('get_kernel_string called')
 
         kernel_source = self.kernel_sources[index]
@@ -235,7 +232,7 @@ class DeviceInterface(object):
             dev = CupyFunctions(device, compiler_options=compiler_options, iterations=iterations, observers=observers)
         elif lang == "OpenCL":
             dev = OpenCLFunctions(device, platform, compiler_options=compiler_options, iterations=iterations, observers=observers)
-        elif lang == "C":
+        elif lang.upper() in ["C", "FORTRAN"]:
             dev = CFunctions(compiler=compiler, compiler_options=compiler_options, iterations=iterations)
         else:
             raise ValueError("Sorry, support for languages other than CUDA, OpenCL, or C is not implemented yet")
@@ -253,6 +250,8 @@ class DeviceInterface(object):
         self.units = dev.units
         self.name = dev.name
         self.max_threads = dev.max_threads
+        self.last_compilation_time = None
+        self.last_verification_time = None
         if not quiet:
             print("Using: " + self.dev.name)
 
@@ -292,6 +291,7 @@ class DeviceInterface(object):
                 logging.debug('benchmark fails due to runtime failure too many resources required')
                 if verbose:
                     print(f"skipping config {util.get_instance_string(instance.params)} reason: too many resources requested for launch")
+                return util.RuntimeFailedConfig()
             else:
                 logging.debug('benchmark encountered runtime failure: ' + str(e))
                 print("Error while benchmarking:", instance.name)
@@ -315,7 +315,7 @@ class DeviceInterface(object):
         #run the kernel
         check = self.run_kernel(func, gpu_args, instance)
         if not check:
-            return True    #runtime failure occured that should be ignored, skip correctness check
+            return    #runtime failure occured that should be ignored, skip correctness check
 
         #retrieve gpu results to host memory
         result_host = []
@@ -341,13 +341,16 @@ class DeviceInterface(object):
 
         if not correct:
             raise RuntimeError("Kernel result verification failed for: " + util.get_config_string(instance.params))
-        return True
 
     def compile_and_benchmark(self, kernel_source, gpu_args, params, kernel_options, tuning_options):
         """ Compile and benchmark a kernel instance based on kernel strings and parameters """
-
+        start_compilation = perf_counter()
         instance_string = util.get_instance_string(params)
 
+        # reset previous timers
+        self.last_compilation_time = None
+        self.last_verification_time = None
+
         logging.debug('compile_and_benchmark ' + instance_string)
         mem_usage = round(resource.getrusage(resource.RUSAGE_SELF).ru_maxrss / 1024.0, 1)
         logging.debug('Memory usage : %2.2f MB', mem_usage)
@@ -355,14 +358,14 @@ class DeviceInterface(object):
         verbose = tuning_options.verbose
 
         instance = self.create_kernel_instance(kernel_source, kernel_options, params, verbose)
-        if instance is None:
-            return None
+        if isinstance(instance, util.ErrorConfig):
+            return instance
 
         try:
             #compile the kernel
             func = self.compile_kernel(instance, verbose)
             if func is None:
-                return None
+                return util.CompilationFailedConfig()
 
             #add shared memory arguments to compiled module
             if kernel_options.smem_args is not None:
@@ -374,11 +377,17 @@ class DeviceInterface(object):
             if kernel_options.texmem_args is not None:
                 self.dev.copy_texture_memory_args(kernel_options.texmem_args)
 
+            # stop compilation stopwatch and convert to miliseconds
+            self.last_compilation_time = 1000 * (perf_counter() - start_compilation)
+
             #test kernel for correctness and benchmark
+            start_verification = perf_counter()
             if tuning_options.answer is not None or tuning_options.verify is not None:
                 self.check_kernel_output(func, gpu_args, instance, tuning_options.answer, tuning_options.atol, tuning_options.verify, verbose)
+            # stop verification stopwatch and convert to miliseconds
+            self.last_verification_time = 1000 * (perf_counter() - start_verification)
 
-            #benchmark
+            # benchmark
             result = self.benchmark(func, gpu_args, instance, verbose)
 
         except Exception as e:
@@ -449,7 +458,7 @@ class DeviceInterface(object):
         if np.prod(threads) > self.dev.max_threads:
             if verbose:
                 print(f"skipping config {util.get_instance_string(params)} reason: too many threads per block")
-            return None
+            return util.InvalidConfig()
 
         #obtain the kernel_string and prepare additional files, if any
         name, kernel_string, temp_files = kernel_source.prepare_list_of_files(kernel_options.kernel_name, params, grid, threads,
@@ -507,23 +516,23 @@ def _default_verify_function(instance, answer, result_host, atol, verbose):
         if answer[i] is not None:    #skip None elements in the answer list
             if isinstance(answer[i], (np.ndarray, cp.ndarray)) and isinstance(arg, (np.ndarray, cp.ndarray)):
                 if answer[i].dtype != arg.dtype:
-                    raise TypeError(f"Element {i} of the expected results list is not of the same dtype as the kernel output: " +
-                                    str(answer[i].dtype) + " != " + str(arg.dtype) + ".")
+                    raise TypeError(f"Element {i} of the expected results list is not of the same dtype as the kernel output: " + str(answer[i].dtype) +
+                                    " != " + str(arg.dtype) + ".")
                 if answer[i].size != arg.size:
-                    raise TypeError(f"Element {i} of the expected results list has a size different from " + "the kernel argument: " +
-                                    str(answer[i].size) + " != " + str(arg.size) + ".")
+                    raise TypeError(f"Element {i} of the expected results list has a size different from " + "the kernel argument: " + str(answer[i].size) +
+                                    " != " + str(arg.size) + ".")
             elif isinstance(answer[i], torch.Tensor) and isinstance(arg, torch.Tensor):
                 if answer[i].dtype != arg.dtype:
-                    raise TypeError(f"Element {i} of the expected results list is not of the same dtype as the kernel output: " +
-                                    str(answer[i].dtype) + " != " + str(arg.dtype) + ".")
+                    raise TypeError(f"Element {i} of the expected results list is not of the same dtype as the kernel output: " + str(answer[i].dtype) +
+                                    " != " + str(arg.dtype) + ".")
                 if answer[i].size() != arg.size():
-                    raise TypeError(f"Element {i} of the expected results list has a size different from " + "the kernel argument: " +
-                                    str(answer[i].size) + " != " + str(arg.size) + ".")
+                    raise TypeError(f"Element {i} of the expected results list has a size different from " + "the kernel argument: " + str(answer[i].size) +
+                                    " != " + str(arg.size) + ".")
 
             elif isinstance(answer[i], np.number) and isinstance(arg, np.number):
                 if answer[i].dtype != arg.dtype:
-                    raise TypeError(f"Element {i} of the expected results list is not the same as the kernel output: " + str(answer[i].dtype) +
-                                    " != " + str(arg.dtype) + ".")
+                    raise TypeError(f"Element {i} of the expected results list is not the same as the kernel output: " + str(answer[i].dtype) + " != " +
+                                    str(arg.dtype) + ".")
             else:
                 #either answer[i] and argument have different types or answer[i] is not a numpy type
                 if not isinstance(answer[i], (np.ndarray, cp.ndarray, torch.Tensor)) or not isinstance(answer[i], np.number):
@@ -572,7 +581,6 @@ def _default_verify_function(instance, answer, result_host, atol, verbose):
     return correct
 
 
-
 #these functions facilitate compiling templated kernels with PyCuda
 def split_argument_list(argument_list):
     """split all arguments in a list into types and names"""
@@ -587,20 +595,24 @@ def split_argument_list(argument_list):
         name_list.append(match.group(2).strip())
     return type_list, name_list
 
+
 def apply_template_typenames(type_list, templated_typenames):
     """replace the typename tokens in type_list with their templated typenames"""
+
     def replace_typename_token(matchobj):
         """function for a whitespace preserving token regex replace"""
         #replace only the match, leaving the whitespace around it as is
         return matchobj.group(1) + templated_typenames[matchobj.group(2)] + matchobj.group(3)
+
     for i, arg_type in enumerate(type_list):
-        for k,v in templated_typenames.items():
+        for k, v in templated_typenames.items():
             #if the templated typename occurs as a token in the string, meaning that it is enclosed in
             #beginning of string or whitespace, and end of string, whitespace or star
             regex = r"(^|\s+)(" + k + r")($|\s+|\*)"
             sub = re.sub(regex, replace_typename_token, arg_type, re.S)
             type_list[i] = sub
 
+
 def get_templated_typenames(template_parameters, template_arguments):
     """based on the template parameters and arguments, create dict with templated typenames"""
     templated_typenames = {}
@@ -610,6 +622,7 @@ def get_templated_typenames(template_parameters, template_arguments):
             templated_typenames[typename] = template_arguments[i]
     return templated_typenames
 
+
 def wrap_templated_kernel(kernel_string, kernel_name):
     """rewrite kernel_string to insert wrapper function for templated kernel"""
     #parse kernel_name to find template_arguments and real kernel name
@@ -626,7 +639,7 @@ def wrap_templated_kernel(kernel_string, kernel_name):
 
     template_parameters = match.group(1).split(',')
     argument_list = match.group(2).split(',')
-    argument_list = [s.strip() for s in argument_list] #remove extra whitespace around 'type name' strings
+    argument_list = [s.strip() for s in argument_list]    #remove extra whitespace around 'type name' strings
 
     type_list, name_list = split_argument_list(argument_list)
 
