@@ -145,7 +145,7 @@ class CupyFunctions:
         kernel_name = kernel_instance.name
 
         compiler_options = self.compiler_options
-        if not any(['--std=' in opt for opt in self.compiler_options]):
+        if not any(['-std=' in opt for opt in self.compiler_options]):
             compiler_options = ['--std=c++11'] + self.compiler_options
 
         options = tuple(compiler_options)
@@ -219,9 +219,10 @@ class CupyFunctions:
             to be numpy objects, such as numpy.ndarray or numpy.int32, and so on.
         :type cmem_args: dict( string: numpy.ndarray, ... )
         """
-        logging.debug('copy_constant_memory_args called')
-        logging.debug('current module: ' + str(self.current_module))
-        raise NotImplementedError('CuPy backend does not yet support constant memory')
+        for k, v in cmem_args.items():
+            symbol = self.current_module.get_global(k)
+            constant_mem = cp.ndarray(v.shape,v.dtype,symbol)
+            constant_mem[:] = cp.asarray(v)
 
     def copy_shared_memory_args(self, smem_args):
         """add shared memory arguments to the kernel"""
@@ -234,7 +235,7 @@ class CupyFunctions:
             device texture memory. See tune_kernel().
         :type texmem_args: dict
         """
-        raise NotImplementedError('CuPy backend does not yet support constant memory')
+        raise NotImplementedError('CuPy backend does not yet support texture memory')
 
     def run_kernel(self, func, gpu_args, threads, grid, stream=None):
         """runs the CUDA kernel passed as 'func'
