@@ -23,16 +23,14 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 """
-from __future__ import print_function
-
-import json
-import os.path
+import sys
 from collections import OrderedDict
-import importlib
 from datetime import datetime
 import logging
-import sys
 import numpy
+from time import perf_counter
+
+from kernel_tuner.integration import get_objective_defaults
 
 import kernel_tuner.util as util
 import kernel_tuner.core as core
@@ -45,7 +43,7 @@ try:
 except ImportError:
     torch = util.TorchPlaceHolder()
 
-from kernel_tuner.strategies import brute_force, random_sample, diff_evo, minimize, basinhopping, genetic_algorithm, mls, pso, simulated_annealing, firefly_algorithm, bayes_opt
+from kernel_tuner.strategies import brute_force, random_sample, diff_evo, minimize, basinhopping, genetic_algorithm, mls, pso, simulated_annealing, firefly_algorithm, bayes_opt, greedy_mls, greedy_ils, ordered_greedy_mls, dual_annealing
 
 strategy_map = {
     "brute_force": brute_force,
@@ -54,6 +52,10 @@ strategy_map = {
     "basinhopping": basinhopping,
     "diff_evo": diff_evo,
     "genetic_algorithm": genetic_algorithm,
+    "greedy_mls": greedy_mls,
+    "ordered_greedy_mls": ordered_greedy_mls,
+    "greedy_ils": greedy_ils,
+    "dual_annealing": dual_annealing,
     "mls": mls,
     "pso": pso,
     "simulated_annealing": simulated_annealing,
@@ -238,99 +240,162 @@ _tuning_options = Options([("tune_params", ("""A dictionary containing the param
                            ("strategy", ("""Specify the strategy to use for searching through the
         parameter space, choose from:
 
-            * "brute_force" (default),
-            * "random_sample", specify: *sample_fraction*,
-            * "minimize" or "basinhopping", specify: *method*,
-            * "diff_evo", specify: *method*.
-            * "genetic_algorithm"
-            * "pso"
-            * "firefly_algorithm"
-            * "simulated_annealing"
-            * "bayes_opt"
+            * "basinhopping" Basin Hopping
+            * "bayes_opt" Bayesian Optimization
+            * "brute_force" (default) iterates through the entire search space
+            * "minimize" uses a local minimization algorithm
+            * "dual annealing" dual annealing
+            * "diff_evo" differential evolution
+            * "firefly_algorithm" firefly algorithm strategy
+            * "genetic_algorithm" a genetic algorithm optimization
+            * "greedy_ils" greedy randomized iterative local search
+            * "greedy_mls" greedy randomized multi-start local search
+            * "mls" best-improvement multi-start local search
+            * "ordered_greedy_mls" multi-start local search that uses a fixed order
+            * "pso" particle swarm optimization
+            * "random_sample" takes a random sample of the search space
+            * "simulated_annealing" simulated annealing strategy
 
-        "brute_force" is the default and iterates over the entire search
-        space.
-
-        "random_sample" can be used to only benchmark a fraction of the
-        search space, specify a *sample_fraction* in the interval [0, 1].
-
-        "minimize" and "basinhopping" strategies use minimizers to
-        limit the search through the parameter space.
-
-        "diff_evo" uses differential evolution.
-
-        "genetic_algorithm" implements a Genetic Algorithm, default
-        setting uses a population size of 20 for 100 generations.
-
-        "pso" implements Particle Swarm Optimization, using the default
-        setting of 20 particles for 100 iterations.
-
-        "firefly_algorithm" implements the Firefly Algorithm, using 20
-        fireflies for 100 iterations.
-
-        "simulated_annealing" uses Simulated Annealing.
-
-        "bayes_opt" uses Bayesian Optimization.
+        Strategy-specific parameters and options are explained under strategy_options.
 
         """, "")),
-                           ("strategy_options", ("""A dict with options for the tuning strategy
+                           ("strategy_options", ("""A dict with options specific to the selected tuning strategy.
 
-        Example usage:
+            All strategies support the following two options:
 
-         * strategy="basinhopping",
-           strategy_options={"method": "BFGS",
-                             "maxiter": 100,
-                             "T": 1.0}
-         * strategy="diff_evo",
-           strategy_options={"method": "best1bin",
-                             "popsize": 20}
-         * strategy="genetic_algorithm",
-           strategy_options={"method": "uniform",
-                             "popsize": 20,
-                             "maxiter": 100}
+            1. "max_fevals": the maximum number of unique valid function evaluations (i.e. compiling and
+            benchmarking a kernel configuration the strategy is allowed to perform as part of the optimization.
+            Note that some strategies implement a default max_fevals of 100.
 
-        strategy="minimize" and strategy="basinhopping", support the following
-        options for "method":
-        "Nelder-Mead", "Powell", "CG", "BFGS", "L-BFGS-B",
-        "TNC", "COBYLA", or "SLSQP". It is also possible to pass a function
-        that implements a custom minimization strategy.
-        The default is "L-BFGS-B".
-        strategy="basinhopping" also supports "T", which is 1.0 by default.
+            2. "time_limit": the maximum amount of time in seconds the strategy is allowed to spent on trying to
+            find the optimal kernel configuration. There is no default time limit.
 
-        strategy="diff_evo" supports the following creation "method" options:
-        "best1bin", "best1exp", "rand1exp", "randtobest1exp", "best2exp",
-        "rand2exp", "randtobest1bin", "best2bin", "rand2bin", "rand1bin".
-        The default is "best1bin".
+            Strategy specific options are explained below:
 
-        strategy="genetic_algorithm" uses "method" to select the crossover
-        method, options are: "single_point", "two_point", "uniform", and
-        "disruptive_uniform".
-        The default is "uniform".
-        Also "mutation_chance" can be set to control the chance of a mutation,
-        which is separately evaluated for each dimension. For example, set
-        to 100 for a probability of 0.01 of a mutation per tunable parameter.
+            * **"basinhopping"**
 
-        strategy="random_sample" supports "fraction" to specify
-        the fraction of the search space to sample in the interval [0,1].
+              * "method", string, any of "Nelder-Mead", "Powell", "CG", "BFGS", "L-BFGS-B", "TNC", "COBYLA", or "SLSQP", default "L-BFGS-B".
 
-        strategy="firefly_algorithm" supports the following parameters:
-        B0 = 1.0, gamma = 1.0, alpha = 0.20.
+              * "T", float, Temperature parameter for the accept or reject criterion, default 1.0.
 
-        strategy="simulated_annealing" supports parameters:
-        T = 1.0, T_min = 0.001, alpha = 0.9.
+            * **"bayes_opt"**
 
-        strategy="bayes_opt" supports acquisition methods: "poi" (default),
-        "ei", "ucb". And parameters, popsize (initial random guesses),
-        maxiter, alpha, kappa, xi.
+              * "covariancekernel", any of "constantrbf", "rbf", "matern32", "matern52", default "matern32".
+
+              * "covariancelengthscale", float, default 1.5.
+
+              * "method": any of "poi", "ei", "lcb", "lcb-srinivas", "multi", "multi-advanced", "multi-fast", default "multi-advanced".
+
+              * "samplingmethod" any of "random", "lhs", default "lhs".
+
+            * **"diff_evo"**
+
+              * "method", string, any of "best1bin", "best1exp", "rand1exp", "randtobest1exp", "best2exp", "rand2exp", "randtobest1bin", "best2bin", "rand2bin", "rand1bin", default "best1bin".
+
+              * "popsize", integer, population size, default 20.
+
+              * "maxiter", integer, number of generations, default 50.
+
+            * **"dual_annealing"**
+
+              * "method", string, any of 'COBYLA','L-BFGS-B','SLSQP','CG','Powell','Nelder-Mead', 'BFGS', 'trust-constr', default "Powell".
+
+            * **"firefly_algorithm"**
+
+              * "alpha", float, alpha parameter, default 0.2.
+
+              * "B0", float, B0 parameter, default 1.0.
+
+              * "gamma", float, gamma parameter, default 1.0.
+
+              * "maxiter", integer, number of generations, default 100.
+
+              * "popsize", integer, population size, default 20.
+
+            * **"genetic_algorithm"**
+
+              * "maxiter", integer, number of generations, default 50.
+
+              * "method", string, crossover method any of "single_point", "two_point", "uniform", "disruptive_uniform", default "uniform".
+
+              * "mutation_chance", integer, specifies the 1 in mutation_chance of a mutation, default 10.
+
+              * "popsize", integer, population size, default 20.
+
+            * **"greedy_ils"**
+
+              * "max_fevals", integer, specifies the maximum allowed number of unique function evaluations, default 100.
+
+              * "neighbor", string, either "Hamming" or "adjacent" specifies how to consider two configurations being neighbors, default Hamming.
+
+              * "no_improvement", integer, number of evaluations to exceed without improvement before restarting, default 50.
+
+              * "randomwalk", float, controls how aggressively to permute the configuration in case no improvement is found, default 0.3.
+
+              * "restart", bool, controls greedyness, i.e. whether to restart from a position as soon as an improvement is found, default True.
+
+            * **"greedy_mls"**
+
+              * "max_fevals", integer, specifies the maximum allowed number of unique function evaluations, default 100.
+
+              * "neighbor", string, either "Hamming" or "adjacent" specifies how to consider two configurations being neighbors, default Hamming.
+
+              * "restart", bool, controls greedyness, i.e. whether to restart from a position as soon as an improvement is found, default True.
+
+            * **"minimize"**
+
+              * "method", string, any of "Nelder-Mead", "Powell", "CG", "BFGS", "L-BFGS-B", "TNC", "COBYLA", or "SLSQP", default "L-BFGS-B".
+
+            * **"mls"**
+
+              * "max_fevals", integer, specifies the maximum allowed number of unique function evaluations, default 100.
+
+              * "neighbor", string, either "Hamming" or "adjacent" specifies how to consider two configurations being neighbors, default Hamming.
+
+            * **"ordered_greedy_mls"**
+
+              * "max_fevals", integer, specifies the maximum allowed number of unique function evaluations, default 100.
+
+              * "neighbor", string, either "Hamming" or "adjacent" specifies how to consider two configurations being neighbors, default Hamming.
+
+              * "order", list, list of integers that describes the order of dimensions in which to look for neighbors, default uses the tune_params specified order.
+
+            * **"pso"**
+
+              * "c1", float, cognitive constant, default 2.0.
+
+              * "c2", float, social constant, default 1.0.
+
+              * "popsize", integer, population size, default 20.
+
+              * "maxiter", integer, number of generations, default 100.
+
+              * "w", float, inertia constant, default 0.5.
+
+            * **"random_sample"**
+
+              * "fraction", float, fraction of the search space to cover in [0,1], default 0.1.
+
+            * **"simulated_annealing"**
+
+              * "alpha", float, alpha parameter, default 0.995.
+
+              * "maxiter", integer, number of iterations within each point of the annealing schedule, default 1.
+
+              * "T", float, starting temperature parameter, default 1.0.
+
+              * "T_min", float, end temperature parameter, default 0.001.
+
+              * "max_fevals", integer, maximum number of unique valid function evaluations, default is set to fit annealing schedule.
 
-        "maxiter" is supported by "minimize", "basinhopping", "diff_evo"
-        "firefly_algorithm", "pso", "genetic_algorithm", "bayes_opt". Note
-        that maxiter generally refers to iterations of the strategy, not
-        the maximum number of function evaluations.
 
     """, "dict")),
                            ("iterations", ("""The number of times a kernel should be executed and
         its execution time measured when benchmarking a kernel, 7 by default.""", "int")),
+                           ("objective", ("""Optimization objective to sort results on, consisting of a string
+            that also occurs in results as a metric or observed quantity, default 'time'""", "string")),
+                           ("objective_higher_is_better", ("""boolean that specifies whether the objective should
+            be maximized (True) or minimized (False), default False.""", "bool")),
                            ("verbose", ("""Sets whether or not to report about configurations that
         were skipped during the search. This could be due to several reasons:
 
@@ -381,20 +446,22 @@ _tune_kernel_docstring = """ Tune a CUDA kernel given a set of tunable parameter
 
 """ % _get_docstring(_kernel_options) + _get_docstring(_tuning_options) + _get_docstring(_device_options)
 
-#"""
 
 
-def tune_kernel(kernel_name, kernel_string, problem_size, arguments, tune_params, grid_div_x=None, grid_div_y=None, grid_div_z=None, restrictions=None,
+def tune_kernel(kernel_name, kernel_source, problem_size, arguments, tune_params, grid_div_x=None, grid_div_y=None, grid_div_z=None, restrictions=None,
                 answer=None, atol=1e-6, verify=None, verbose=False, lang=None, device=0, platform=0, smem_args=None, cmem_args=None, texmem_args=None,
                 compiler=None, compiler_options=None, log=None, iterations=7, block_size_names=None, quiet=False, strategy=None, strategy_options=None,
-                cache=None, metrics=None, simulation_mode=False, observers=None):
-
+                cache=None, metrics=None, simulation_mode=False, observers=None, objective=None, objective_higher_is_better=None):
+    start_overhead_time = perf_counter()
     if log:
         logging.basicConfig(filename=kernel_name + datetime.now().strftime('%Y%m%d-%H:%M:%S') + '.log', level=log)
 
-    kernel_source = core.KernelSource(kernel_name, kernel_string, lang)
+    kernelsource = core.KernelSource(kernel_name, kernel_source, lang)
 
-    _check_user_input(kernel_name, kernel_source, arguments, block_size_names)
+    _check_user_input(kernel_name, kernelsource, arguments, block_size_names)
+
+    # default objective if none is specified
+    objective, objective_higher_is_better = get_objective_defaults(objective, objective_higher_is_better)
 
     # check for forbidden names in tune parameters
     util.check_tune_params_list(tune_params)
@@ -402,6 +469,13 @@ def tune_kernel(kernel_name, kernel_string, problem_size, arguments, tune_params
     # check whether block_size_names are used as expected
     util.check_block_size_params_names_list(block_size_names, tune_params)
 
+    # ensure there is always at least three names
+    util.append_default_block_size_names(block_size_names)
+
+    # if there are string in the restrictions, parse them to functions (increases restrictions check performance significantly)
+    if isinstance(restrictions, list) and len(restrictions) > 0 and any(isinstance(restriction, str) for restriction in restrictions):
+        restrictions = util.parse_restrictions(restrictions)
+
     if iterations < 1:
         raise ValueError("Iterations should be at least one!")
 
@@ -411,6 +485,11 @@ def tune_kernel(kernel_name, kernel_string, problem_size, arguments, tune_params
     tuning_options = Options([(k, opts[k]) for k in _tuning_options.keys()])
     device_options = Options([(k, opts[k]) for k in _device_options.keys()])
     tuning_options["snap"] = True
+    tuning_options["unique_results"] = {}
+    if strategy_options and "max_fevals" in strategy_options:
+        tuning_options["max_fevals"] = strategy_options["max_fevals"]
+    if strategy_options and "time_limit" in strategy_options:
+        tuning_options["time_limit"] = strategy_options["time_limit"]
 
     logging.debug('tune_kernel called')
     logging.debug('kernel_options: %s', util.get_config_string(kernel_options))
@@ -423,39 +502,39 @@ def tune_kernel(kernel_name, kernel_string, problem_size, arguments, tune_params
         else:
             raise ValueError("Strategy %s not recognized" % strategy)
 
-        #make strategy_options into an Options object
+        # make strategy_options into an Options object
         if tuning_options.strategy_options:
             if not isinstance(strategy_options, Options):
                 tuning_options.strategy_options = Options(strategy_options)
 
-            #select strategy based on user options
+            # select strategy based on user options
             if "fraction" in tuning_options.strategy_options and not tuning_options.strategy == 'random_sample':
                 raise ValueError('It is not possible to use fraction in combination with strategies other than "random_sample". ' \
                                  'Please set strategy="random_sample", when using "fraction" in strategy_options')
 
-            #check if method is supported by the selected strategy
+            # check if method is supported by the selected strategy
             if "method" in tuning_options.strategy_options:
                 method = tuning_options.strategy_options.method
                 if not method in strategy.supported_methods:
                     raise ValueError('Method %s is not supported for strategy %s' % (method, tuning_options.strategy))
 
-        #if no strategy_options dict has been passed, create empty dictionary
+        # if no strategy_options dict has been passed, create empty dictionary
         else:
             tuning_options.strategy_options = Options({})
 
-    #if no strategy selected
+    # if no strategy selected
     else:
         strategy = brute_force
 
     # select the runner for this job based on input
     selected_runner = SimulationRunner if simulation_mode is True else SequentialRunner
-    with selected_runner(kernel_source, kernel_options, device_options, iterations, observers) as runner:
+    with selected_runner(kernelsource, kernel_options, device_options, iterations, observers) as runner:
 
-        #the user-specified function may or may not have an optional atol argument;
-        #we normalize it so that it always accepts atol.
+        # the user-specified function may or may not have an optional atol argument;
+        # we normalize it so that it always accepts atol.
         tuning_options.verify = util.normalize_verify_function(tuning_options.verify)
 
-        #process cache
+        # process cache
         if cache:
             if cache[-5:] != ".json":
                 cache += ".json"
@@ -465,13 +544,14 @@ def tune_kernel(kernel_name, kernel_string, problem_size, arguments, tune_params
             tuning_options.cache = {}
             tuning_options.cachefile = None
 
-        #call the strategy to execute the tuning process
+        # call the strategy to execute the tuning process
+        tuning_options["start_time"] = perf_counter()
         results, env = strategy.tune(runner, kernel_options, device_options, tuning_options)
 
-        #finished iterating over search space
+        # finished iterating over search space
         if not device_options.quiet:
-            if results:    #checks if results is not empty
-                best_config = min(results, key=lambda x: x['time'])
+            if results:    # checks if results is not empty
+                best_config = util.get_best_config(results, objective, objective_higher_is_better)
                 units = getattr(runner, "units", None)
                 print("best performing configuration:")
                 util.print_config_output(tune_params, best_config, device_options.quiet, metrics, units)
@@ -481,6 +561,9 @@ def tune_kernel(kernel_name, kernel_string, problem_size, arguments, tune_params
         if cache:
             util.close_cache(cache)
 
+    # get the seperate timings for the benchmarking process
+    overhead_time = 1000 * (perf_counter() - start_overhead_time)
+    env = util.get_total_timings(results, env, overhead_time)
     return results, env
 
 
@@ -515,15 +598,15 @@ _run_kernel_docstring = """Compile and run a single kernel
 """ % _get_docstring(_kernel_options) + _get_docstring(_device_options)
 
 
-def run_kernel(kernel_name, kernel_string, problem_size, arguments, params, grid_div_x=None, grid_div_y=None, grid_div_z=None, lang=None, device=0, platform=0,
+def run_kernel(kernel_name, kernel_source, problem_size, arguments, params, grid_div_x=None, grid_div_y=None, grid_div_z=None, lang=None, device=0, platform=0,
                smem_args=None, cmem_args=None, texmem_args=None, compiler=None, compiler_options=None, block_size_names=None, quiet=False, log=None):
 
     if log:
         logging.basicConfig(filename=kernel_name + datetime.now().strftime('%Y%m%d-%H:%M:%S') + '.log', level=log)
 
-    kernel_source = core.KernelSource(kernel_name, kernel_string, lang)
+    kernelsource = core.KernelSource(kernel_name, kernel_source, lang)
 
-    _check_user_input(kernel_name, kernel_source, arguments, block_size_names)
+    _check_user_input(kernel_name, kernelsource, arguments, block_size_names)
 
     #sort options into separate dicts
     opts = locals()
@@ -531,7 +614,7 @@ def run_kernel(kernel_name, kernel_string, problem_size, arguments, params, grid
     device_options = Options([(k, opts[k]) for k in _device_options.keys()])
 
     #detect language and create the right device function interface
-    with core.DeviceInterface(kernel_source, iterations=1, **device_options) as dev:
+    with core.DeviceInterface(kernelsource, iterations=1, **device_options) as dev:
 
         #move data to the GPU
         gpu_args = dev.ready_argument_list(arguments)
@@ -539,9 +622,9 @@ def run_kernel(kernel_name, kernel_string, problem_size, arguments, params, grid
         instance = None
         try:
             #create kernel instance
-            instance = dev.create_kernel_instance(kernel_source, kernel_options, params, False)
+            instance = dev.create_kernel_instance(kernelsource, kernel_options, params, False)
             if instance is None:
-                raise Exception("cannot create kernel instance, too many threads per block")
+                raise RuntimeError("cannot create kernel instance, too many threads per block")
 
             # see if the kernel arguments have correct type
             util.check_argument_list(instance.name, instance.kernel_string, arguments)
@@ -549,7 +632,7 @@ def run_kernel(kernel_name, kernel_string, problem_size, arguments, params, grid
             #compile the kernel
             func = dev.compile_kernel(instance, False)
             if func is None:
-                raise Exception("cannot compile kernel, too much shared memory used")
+                raise RuntimeError("cannot compile kernel, too much shared memory used")
 
             #add shared memory arguments to compiled module
             if smem_args is not None:
@@ -567,7 +650,7 @@ def run_kernel(kernel_name, kernel_string, problem_size, arguments, params, grid
 
         #run the kernel
         if not dev.run_kernel(func, gpu_args, instance):
-            raise Exception("runtime error occured, too many resources requested")
+            raise RuntimeError("runtime error occured, too many resources requested")
 
         #copy data in GPU memory back to the host
         results = []
@@ -586,9 +669,9 @@ def run_kernel(kernel_name, kernel_string, problem_size, arguments, params, grid
 run_kernel.__doc__ = _run_kernel_docstring
 
 
-def _check_user_input(kernel_name, kernel_source, arguments, block_size_names):
+def _check_user_input(kernel_name, kernelsource, arguments, block_size_names):
     # see if the kernel arguments have correct type
-    kernel_source.check_argument_lists(kernel_name, arguments)
+    kernelsource.check_argument_lists(kernel_name, arguments)
 
     # check for types and length of block_size_names
     util.check_block_size_names(block_size_names)
