@@ -1,5 +1,7 @@
+import subprocess
 import time
 import numpy as np
+import re
 
 from kernel_tuner.observers import BenchmarkObserver
 
@@ -11,11 +13,13 @@ except ImportError:
 class nvml():
     """Class that gathers the NVML functionality for one device"""
 
-    def __init__(self, device_id=0):
+    def __init__(self, device_id=0, nvidia_smi_fallback='nvidia-smi'):
         """Create object to control device using NVML"""
 
         pynvml.nvmlInit()
         self.dev = pynvml.nvmlDeviceGetHandleByIndex(device_id)
+        self.id = device_id
+        self.nvidia_smi = nvidia_smi_fallback
 
         try:
             self._pwr_limit = pynvml.nvmlDeviceGetPowerManagementLimit(self.dev)
@@ -82,7 +86,15 @@ class nvml():
     def pwr_limit(self, new_limit):
         if not self.pwr_constraints[0] <= new_limit <= self.pwr_constraints[1]:
             raise ValueError(f"Power limit {new_limit} out of range [{self.pwr_constraints[0]}, {self.pwr_constraints[1]}]")
-        pynvml.nvmlDeviceSetPowerManagementLimit(self.dev, new_limit)
+        if new_limit == pynvml.nvmlDeviceGetPowerManagementLimit(self.dev):
+            return
+        try:
+            pynvml.nvmlDeviceSetPowerManagementLimit(self.dev, new_limit)
+        except pynvml.NVMLError_NoPermission:
+            if self.nvidia_smi:
+                new_limit_watt = int(new_limit / 1000.0) # nvidia-smi expects Watts rather than milliwatts
+                args = ["sudo", self.nvidia_smi, "-i", str(self.id), "--power-limit="+str(new_limit_watt)]
+                out = subprocess.run(args, check=True)
         self._pwr_limit = pynvml.nvmlDeviceGetPowerManagementLimit(self.dev)
 
     @property
@@ -103,7 +115,13 @@ class nvml():
             raise ValueError("Illegal value for memory clock")
         if not gr_clock in self.supported_gr_clocks[mem_clock]:
             raise ValueError("Graphics clock incompatible with memory clock")
-        pynvml.nvmlDeviceSetApplicationsClocks(self.dev, mem_clock, gr_clock)
+        try:
+            pynvml.nvmlDeviceSetApplicationsClocks(self.dev, mem_clock, gr_clock)
+        except pynvml.NVMLError_NoPermission:
+            if self.nvidia_smi:
+                args = ["sudo", self.nvidia_smi, "-i", str(self.id), "--applications-clocks="+str(mem_clock)+","+str(gr_clock)]
+                out = subprocess.run(args, check=True)
+
         self._gr_clock = pynvml.nvmlDeviceGetApplicationsClock(self.dev, pynvml.NVML_CLOCK_GRAPHICS)
         self._sm_clock = pynvml.nvmlDeviceGetApplicationsClock(self.dev, pynvml.NVML_CLOCK_SM)
         self._mem_clock = pynvml.nvmlDeviceGetApplicationsClock(self.dev, pynvml.NVML_CLOCK_MEM)
@@ -115,7 +133,8 @@ class nvml():
 
     @gr_clock.setter
     def gr_clock(self, new_clock):
-        self.set_clocks(self._mem_clock, new_clock)
+        if new_clock != pynvml.nvmlDeviceGetApplicationsClock(self.dev, pynvml.NVML_CLOCK_GRAPHICS):
+            self.set_clocks(self._mem_clock, new_clock)
 
     @property
     def mem_clock(self):
@@ -124,7 +143,8 @@ class nvml():
 
     @mem_clock.setter
     def mem_clock(self, new_clock):
-        self.set_clocks(new_clock, self._gr_clock)
+        if new_clock != pynvml.nvmlDeviceGetApplicationsClock(self.dev, pynvml.NVML_CLOCK_MEM):
+            self.set_clocks(new_clock, self._gr_clock)
 
     @property
     def temperature(self):
@@ -148,13 +168,21 @@ class nvml():
         """Return current power usage in milliwatts"""
         return pynvml.nvmlDeviceGetPowerUsage(self.dev)
 
+    def gr_voltage(self):
+        """Return current graphics voltage in millivolts"""
+        args = ["nvidia-smi", "-i", str(self.id), "-q",  "-d", "VOLTAGE"]
+        result = subprocess.run(args, check=True, capture_output=True)
+        m = re.search(r"(\d+\.\d+) mV", result.stdout.decode())
+        voltage = float(m.group(1))
+        return voltage
 
 class NVMLObserver(BenchmarkObserver):
     """ Observer that measures time using CUDA events during benchmarking """
-    def __init__(self, observables, device=0):
-        self.nvml = nvml(device)
+    def __init__(self, observables, device=0, save_all=False, nvidia_smi_fallback=None):
+        self.nvml = nvml(device, nvidia_smi_fallback=nvidia_smi_fallback)
+        self.save_all = save_all
 
-        supported = ["power_readings", "nvml_power", "nvml_energy", "core_freq", "mem_freq", "temperature"]
+        supported = ["power_readings", "nvml_power", "nvml_energy", "core_freq", "mem_freq", "temperature", "gr_voltage"]
         for obs in observables:
             if not obs in supported:
                 raise ValueError(f"Observable {obs} not in supported: {supported}")
@@ -166,6 +194,11 @@ class NVMLObserver(BenchmarkObserver):
             self.measure_power = True
             self.power_readings = []
 
+        self.record_gr_voltage = False
+        if "gr_voltage" in observables:
+            self.record_gr_voltage = True
+            self.gr_voltage_readings = []
+
         self.results = dict()
         for obs in observables:
             self.results[obs] = []
@@ -173,6 +206,8 @@ class NVMLObserver(BenchmarkObserver):
     def before_start(self):
         if self.measure_power:
             self.power_readings = []
+        if self.record_gr_voltage:
+            self.gr_voltage_readings = []
 
     def after_start(self):
         self.t0 = time.time()
@@ -180,6 +215,8 @@ class NVMLObserver(BenchmarkObserver):
     def during(self):
         if self.measure_power:
             self.power_readings.append([time.time()-self.t0, self.nvml.pwr_usage()])
+        if self.record_gr_voltage:
+            self.gr_voltage_readings.append([time.time()-self.t0, self.nvml.gr_voltage()])
 
     def after_finish(self):
         if self.measure_power:
@@ -213,12 +250,22 @@ class NVMLObserver(BenchmarkObserver):
         if "mem_freq" in self.observables:
             self.results["mem_freq"].append(self.nvml.mem_clock)
 
+        if "gr_voltage" in self.observables:
+            gr_voltage_readings = self.gr_voltage_readings
+            gr_voltage_readings = [[0.0, gr_voltage_readings[0][1]]] + gr_voltage_readings
+            gr_voltage_readings = gr_voltage_readings + [[execution_time, gr_voltage_readings[-1][1]]]
+            self.results["gr_voltage"].append(np.average(gr_voltage_readings[:][:][1])) #time in s, graphics voltage in millivolts
+
     def get_results(self):
         averaged_results = dict()
 
         #return averaged results, except for power_readings
         for obs in self.observables:
             if not obs == "power_readings":
+                #save all information, if the user requested
+                if self.save_all:
+                    averaged_results[obs + "s"] = self.results[obs]
+                #save averaged results, default
                 averaged_results[obs] = np.average(self.results[obs])
         if "power_readings" in self.observables:
             averaged_results["power_readings"] = self.results["power_readings"].copy()
