@@ -1,10 +1,9 @@
 """ The default runner for sequentially tuning the parameter space """
-from __future__ import print_function
-
 from collections import OrderedDict
 import logging
+from time import perf_counter
 
-from kernel_tuner.util import get_config_string, store_cache, process_metrics, print_config_output
+from kernel_tuner.util import get_config_string, store_cache, process_metrics, print_config_output, ErrorConfig
 from kernel_tuner.core import DeviceInterface
 
 
@@ -35,10 +34,9 @@ class SequentialRunner(object):
         self.units = self.dev.units
         self.quiet = device_options.quiet
         self.kernel_source = kernel_source
-
         self.warmed_up = False
-
         self.simulation_mode = False
+        self.last_strategy_start_time = perf_counter()
 
         #move data to the GPU
         self.gpu_args = self.dev.ready_argument_list(kernel_options.arguments)
@@ -80,27 +78,28 @@ class SequentialRunner(object):
 
             #check if element is in the cache
             x_int = ",".join([str(i) for i in element])
-            if tuning_options.cache:
-                if x_int in tuning_options.cache:
-                    results.append(tuning_options.cache[x_int])
-                    continue
-
-            result = self.dev.compile_and_benchmark(self.kernel_source, self.gpu_args, params, kernel_options, tuning_options)
-            if result is None:
-                logging.debug('received benchmark result is None, kernel configuration was skipped silently due to compile or runtime failure')
-                params.update({ "time": 1e20 })
-                store_cache(x_int, params, tuning_options)
+            if tuning_options.cache and x_int in tuning_options.cache:
+                results.append(tuning_options.cache[x_int])
                 continue
 
-            #print and append to results
-            if isinstance(result, dict):
-                time = result["time"]
+            result = self.dev.compile_and_benchmark(self.kernel_source, self.gpu_args, params, kernel_options, tuning_options)
+
+            if self.dev.last_compilation_time is not None:
+                params['compile_time'] = self.dev.last_compilation_time
+            if self.dev.last_verification_time is not None:
+                params['verification_time'] = self.dev.last_verification_time
+
+            if isinstance(result, ErrorConfig):
+                logging.debug('kernel configuration was skipped silently due to compile or runtime failure')
+                params.update({ tuning_options.objective: result })
+                store_cache(x_int, params, tuning_options)
+                results.append(params)
+                continue
+
+            # print and append to results
+            if not isinstance(result, dict):
+                params[tuning_options.objective] = result
             else:
-                time = result
-
-            params['time'] = time
-
-            if isinstance(result, dict):
                 params.update(result)
 
             if tuning_options.metrics:
