@@ -1,9 +1,9 @@
 """ The differential evolution strategy that optimizes the search through the parameter space """
-from __future__ import print_function
-
 from scipy.optimize import differential_evolution
 
-from kernel_tuner.strategies.minimize import get_bounds, _cost_func
+from kernel_tuner.searchspace import Searchspace
+from kernel_tuner.strategies.minimize import get_bounds, _cost_func, scale_from_params
+from kernel_tuner import util
 
 supported_methods = ["best1bin", "best1exp", "rand1exp", "randtobest1exp",
                      "best2exp", "rand2exp", "randtobest1bin", "best2bin", "rand2bin", "rand1bin"]
@@ -36,6 +36,8 @@ def tune(runner, kernel_options, device_options, tuning_options):
     results = []
 
     method = tuning_options.strategy_options.get("method", "best1bin")
+    popsize = tuning_options.strategy_options.get("popsize", 20)
+    maxiter = tuning_options.strategy_options.get("maxiter", 50)
 
     tuning_options["scaling"] = False
     # build a bounds array as needed for the optimizer
@@ -43,11 +45,20 @@ def tune(runner, kernel_options, device_options, tuning_options):
 
     args = (kernel_options, tuning_options, runner, results)
 
-    # call the differential evolution optimizer
-    opt_result = differential_evolution(_cost_func, bounds, args, maxiter=1,
-                                        polish=False, strategy=method, disp=tuning_options.verbose)
+    # ensure particles start from legal points
+    searchspace = Searchspace(tuning_options, runner.dev.max_threads)
+    population = list(list(p) for p in searchspace.get_random_sample(popsize))
 
-    if tuning_options.verbose:
+    # call the differential evolution optimizer
+    opt_result = None
+    try:
+        opt_result = differential_evolution(_cost_func, bounds, args, maxiter=maxiter, popsize=popsize, init=population,
+                                        polish=False, strategy=method, disp=tuning_options.verbose)
+    except util.StopCriterionReached as e:
+        if tuning_options.verbose:
+            print(e)
+
+    if opt_result and tuning_options.verbose:
         print(opt_result.message)
 
     return results, runner.dev.get_environment()
