@@ -1,12 +1,10 @@
 """ Iterate over a random sample of the parameter space """
-from __future__ import print_function
-
-import itertools
-import numpy
+import numpy as np
 
+from kernel_tuner.searchspace import Searchspace
+from kernel_tuner.strategies.minimize import _cost_func
 from kernel_tuner import util
 
-
 def tune(runner, kernel_options, device_options, tuning_options):
     """ Tune a random sample of sample_fraction fraction in the parameter space
 
@@ -25,31 +23,35 @@ def tune(runner, kernel_options, device_options, tuning_options):
     :type tuning_options: kernel_tuner.interface.Options
 
     :returns: A list of dictionaries for executed kernel configurations and their
-        execution times. And a dictionary that contains a information
+        execution times. And a dictionary that contains information
         about the hardware/software environment on which the tuning took place.
     :rtype: list(dict()), dict()
 
     """
 
-    tune_params = tuning_options.tune_params
+    tuning_options["scaling"] = False
 
+    # create the search space
+    searchspace = Searchspace(tuning_options, runner.dev.max_threads)
+
+    # get the samples
     fraction = tuning_options.strategy_options.get("fraction", 0.1)
+    num_samples = int(np.ceil(searchspace.size * fraction))
 
-    # compute cartesian product of all tunable parameters
-    parameter_space = itertools.product(*tune_params.values())
+    # override if max_fevals is specified
+    if "max_fevals" in tuning_options:
+        num_samples = tuning_options.max_fevals
 
-    # check for search space restrictions
-    if tuning_options.restrictions is not None:
-        parameter_space = filter(lambda p: util.check_restrictions(tuning_options.restrictions, p, tune_params.keys(), tuning_options.verbose), parameter_space)
+    samples = searchspace.get_random_sample(num_samples)
 
-    # reduce parameter space to a random sample using sample_fraction
-    parameter_space = numpy.array(list(parameter_space))
-    size = len(parameter_space)
-    fraction = int(numpy.ceil(size * fraction))
-    sample_indices = numpy.random.choice(range(size), size=fraction, replace=False)
-    parameter_space = parameter_space[sample_indices]
+    results = []
 
-    # call the runner
-    results, env = runner.run(parameter_space, kernel_options, tuning_options)
+    for sample in samples:
+        try:
+            _cost_func(sample, kernel_options, tuning_options, runner, results, check_restrictions=False)
+        except util.StopCriterionReached as e:
+            if tuning_options.verbose:
+                print(e)
+            return results, runner.dev.get_environment()
 
-    return results, env
+    return results, runner.dev.get_environment()
