@@ -1,28 +1,54 @@
 """ Module for kernel tuner utility functions """
-from __future__ import print_function
-
+import time
+from inspect import signature
 import json
 from collections import OrderedDict
 import os
+import sys
 import errno
 import tempfile
 import logging
 import warnings
 import re
+from types import FunctionType
 
 import numpy as np
+from constraint import Constraint, AllDifferentConstraint, AllEqualConstraint, MaxSumConstraint, ExactSumConstraint, MinSumConstraint, InSetConstraint, NotInSetConstraint, SomeInSetConstraint, SomeNotInSetConstraint, FunctionConstraint
 try:
     import cupy as cp
 except ImportError:
     cp = np
 
 
+# number of special values to insert when a configuration cannot be measured
+
+class ErrorConfig(str):
+    def __str__(self): return self.__class__.__name__
+    def __repr__(self): return self.__class__.__name__
+
+class InvalidConfig(ErrorConfig):
+    pass
+
+class CompilationFailedConfig(ErrorConfig):
+    pass
+
+class RuntimeFailedConfig(ErrorConfig):
+    pass
+
+
 class TorchPlaceHolder():
 
     def __init__(self):
         self.Tensor = Exception    #using Exception here as a type that will never be among kernel arguments
 
 
+class SkippableFailure(Exception):
+    """Exception used to raise when compiling or launching a kernel fails for a reason that can be expected"""
+
+class StopCriterionReached(Exception):
+    """Exception thrown when a stop criterion has been reached"""
+
+
 try:
     import torch
 except ImportError:
@@ -44,7 +70,9 @@ def check_argument_type(dtype, kernel_argument):
         "int64": ["long", "int64_t"],
         "float16": ["half"],
         "float32": ["float"],
-        "float64": ["double"]
+        "float64": ["double"],
+        "complex64": ["float2"],
+        "complex128": ["double2"]
     }
     if dtype in types_map:
         return any([substr in kernel_argument for substr in types_map[dtype]])
@@ -86,7 +114,14 @@ def check_argument_list(kernel_name, kernel_string, args):
             return
     for errors in collected_errors:
         warnings.warn(errors[0], UserWarning)
-        # raise TypeError(errors[0])
+
+
+def check_stop_criterion(to):
+    """ checks if max_fevals is reached or time limit is exceeded """
+    if "max_fevals" in to and len(to.unique_results) >= to.max_fevals:
+        raise StopCriterionReached("max_fevals reached")
+    if "time_limit" in to and (time.perf_counter() - to.start_time > to.time_limit):
+        raise StopCriterionReached("time limit exceeded")
 
 
 def check_tune_params_list(tune_params):
@@ -106,10 +141,14 @@ def check_block_size_names(block_size_names):
             raise ValueError("block_size_names should not contain more than 3 names!")
         if not all([isinstance(name, "".__class__) for name in block_size_names]):
             raise ValueError("block_size_names should contain only strings!")
-        # ensure there is always at least three names
-        for i, name in enumerate(default_block_size_names):
-            if len(block_size_names) < i + 1:
-                block_size_names.append(name)
+
+
+def append_default_block_size_names(block_size_names):
+    if block_size_names is None:
+        return
+    for i, name in enumerate(default_block_size_names):
+        if len(block_size_names) < i + 1:
+            block_size_names.append(name)
 
 
 def check_block_size_params_names_list(block_size_names, tune_params):
@@ -122,17 +161,24 @@ def check_block_size_params_names_list(block_size_names, tune_params):
             warnings.warn("None of the tunable parameters specify thread block dimensions!", UserWarning)
 
 
-def check_restrictions(restrictions, element, keys, verbose):
+def check_restrictions(restrictions, params: dict, verbose: bool):
     """ check whether a specific instance meets the search space restrictions """
-    params = OrderedDict(zip(keys, element))
     valid = True
     if callable(restrictions):
         valid = restrictions(params)
     else:
         for restrict in restrictions:
             try:
-                if not eval(replace_param_occurrences(restrict, params)):
+                # if it's a python-constraint, convert to function and execute
+                if isinstance(restrict, Constraint):
+                    restrict = convert_constraint_restriction(restrict)
+                    if not restrict(params.values()):
+                        valid = False
+                        break
+                # if it's a string, fill in the parameters and evaluate
+                elif not eval(replace_param_occurrences(restrict, params)):
                     valid = False
+                    break
             except ZeroDivisionError:
                 pass
     if not valid and verbose:
@@ -140,14 +186,48 @@ def check_restrictions(restrictions, element, keys, verbose):
     return valid
 
 
+def convert_constraint_restriction(restrict: Constraint):
+    """ Convert the python-constraint to a function for backwards compatibility """
+    if isinstance(restrict, FunctionConstraint):
+        f_restrict = lambda p: restrict._func(*p)
+    elif isinstance(restrict, AllDifferentConstraint):
+        f_restrict = lambda p: len(set(p)) == len(p)
+    elif isinstance(restrict, AllEqualConstraint):
+        f_restrict = lambda p: all(x == p[0] for x in p)
+    elif isinstance(restrict, MaxProdConstraint):
+        f_restrict = lambda p: np.prod(p) <= restrict._exactsum
+    elif isinstance(restrict, MaxSumConstraint):
+        f_restrict = lambda p: sum(p) <= restrict._exactsum
+    elif isinstance(restrict, ExactSumConstraint):
+        f_restrict = lambda p: sum(p) == restrict._exactsum
+    elif isinstance(restrict, MinSumConstraint):
+        f_restrict = lambda p: sum(p) >= restrict._exactsum
+    elif isinstance(restrict, (InSetConstraint, NotInSetConstraint, SomeInSetConstraint, SomeNotInSetConstraint)):
+        raise NotImplementedError(
+            f"Restriction of the type {type(restrict)} is explicitely not supported in backwards compatibility mode, because the behaviour is too complex. Please rewrite this constraint to a function to use it with this algorithm."
+        )
+    else:
+        raise TypeError(f"Unrecognized restriction {restrict}")
+    return f_restrict
+
+
+def check_thread_block_dimensions(params, max_threads, block_size_names=None):
+    """ check on maximum thread block dimensions """
+    dims = get_thread_block_dimensions(params, block_size_names)
+    return np.prod(dims) <= max_threads
+
+
 def config_valid(config, tuning_options, max_threads):
     """ combines restrictions and a check on the max thread block dimension to check config validity """
     legal = True
-    if tuning_options.restrictions:
-        legal = check_restrictions(tuning_options.restrictions, config, tuning_options.tune_params.keys(), False)
     params = OrderedDict(zip(tuning_options.tune_params.keys(), config))
-    dims = get_thread_block_dimensions(params, tuning_options.get("block_size_names", None))
-    return legal and np.prod(dims) <= max_threads
+    if tuning_options.restrictions:
+        legal = check_restrictions(tuning_options.restrictions, params, False)
+        if not legal:
+            return False
+    block_size_names = tuning_options.get("block_size_names", None)
+    valid_thread_block_dimensions = check_thread_block_dimensions(params, max_threads, block_size_names)
+    return legal and valid_thread_block_dimensions
 
 
 def delete_temp_file(filename):
@@ -170,6 +250,14 @@ def detect_language(kernel_string):
     return lang
 
 
+def get_best_config(results, objective, objective_higher_is_better=False):
+    """ Returns the best configuration from a list of results according to some objective """
+    func = max if objective_higher_is_better else min
+    ignore_val = sys.float_info.max if not objective_higher_is_better else -sys.float_info.max
+    best_config = func(results, key=lambda x: x[objective] if isinstance(x[objective], float) else ignore_val)
+    return best_config
+
+
 def get_config_string(params, keys=None, units=None):
     """ return a compact string representation of a measurement """
 
@@ -186,7 +274,8 @@ def get_config_string(params, keys=None, units=None):
     for k, v in params.items():
         if k in keys:
             unit = ""
-            if isinstance(units, dict):    # check if not None not enough, units could be mocked which causes errors
+            # check if units not None not enough, units could be mocked which causes errors
+            if isinstance(units, dict) and not isinstance(v, ErrorConfig):
                 unit = units.get(k, "")
             compact_str_items.append(k + "=" + compact_number(v) + unit)
     # and finally join them
@@ -309,6 +398,39 @@ def get_thread_block_dimensions(params, block_size_names=None):
     return (int(block_size_x), int(block_size_y), int(block_size_z))
 
 
+def get_total_timings(results, env, overhead_time):
+    """ Sum all timings and put their totals in the env """
+    total_framework_time = 0
+    total_strategy_time = 0
+    total_compile_time = 0
+    total_verification_time = 0
+    total_kernel_time = 0
+    if results:
+        for result in results:
+            if 'framework_time' not in result or 'strategy_time' not in result or 'compile_time' not in result or 'verification_time' not in result:
+                #warnings.warn("No detailed timings in results")
+                return env
+            total_framework_time += result['framework_time']
+            total_strategy_time += result['strategy_time']
+            total_compile_time += result['compile_time']
+            total_verification_time += result['verification_time']
+            total_kernel_time += sum(result['times']) if 'times' in result.keys() else 0
+
+    # add the seperate times to the environment dict
+    env['total_framework_time'] = total_framework_time
+    env['total_strategy_time'] = total_strategy_time
+    env['total_compile_time'] = total_compile_time
+    env['total_verification_time'] = total_verification_time
+    env['total_kernel_time'] = total_kernel_time
+    env['overhead_time'] = overhead_time - (total_framework_time + total_strategy_time + total_compile_time + total_verification_time + total_kernel_time)
+    return env
+
+
+def print_config(config, tuning_options, runner):
+    """print the configuration string with tunable parameters and benchmark results"""
+    print_config_output(tuning_options.tune_params, config, runner.quiet, tuning_options.metrics, runner.units)
+
+
 def print_config_output(tune_params, params, quiet, metrics, units):
     """print the configuration string with tunable parameters and benchmark results"""
     print_keys = list(tune_params.keys()) + ["time"]
@@ -422,6 +544,10 @@ def prepare_kernel_string(kernel_name, kernel_string, params, grid, threads, blo
     """
     logging.debug('prepare_kernel_string called for %s', kernel_name)
 
+    # since we insert defines above the original kernel code, the line numbers will be incorrect
+    # the following preprocessor directive informs the compiler that lines should be counted from 1
+    kernel_string = "#line 1\n" + kernel_string
+
     grid_dim_names = ["grid_size_x", "grid_size_y", "grid_size_z"]
     for i, g in enumerate(grid):
         kernel_string = f"#define {grid_dim_names[i]} {g}\n" + kernel_string
@@ -443,8 +569,6 @@ def prepare_kernel_string(kernel_name, kernel_string, params, grid, threads, blo
 
     # also insert kernel_tuner token
     kernel_string = "#define kernel_tuner 1\n" + kernel_string
-    # name = kernel_name + "_" + get_instance_string(params)
-    # kernel_string = kernel_string.replace(kernel_name, name)
 
     return name, kernel_string
 
@@ -473,7 +597,6 @@ def setup_block_and_grid(problem_size, grid_div, params, block_size_names=None):
 
 def write_file(filename, string):
     """dump the contents of string to a file called filename"""
-    import sys
     # ugly fix, hopefully we can find a better one
     if sys.version_info[0] >= 3:
         with open(filename, 'w', encoding="utf-8") as f:
@@ -492,44 +615,41 @@ def normalize_verify_function(v):
 
     Undefined behaviour if the passed function does not match the required signatures.
     """
-
     # python 3.3+
-    def _has_kw_argument_sig(func, name):
-        from inspect import signature
+    def has_kw_argument(func, name):
         sig = signature(func)
         return name in sig.parameters
 
-    # python 3.0+
-    def _has_kw_argument_fullarg(func, name):
-        from inspect import getfullargspec
-        spec = getfullargspec(func)
-        return (name in spec.args) or (name in spec.kwonlyargs)
-
-    # python 2.6+
-    def _has_kw_argument_arg(func, name):
-        from inspect import getargspec
-        spec = getargspec(func)
-        return name in spec.args
-
     if v is None:
         return None
 
-    import inspect
-
-    if hasattr(inspect, 'signature'):
-        has_kw_argument = _has_kw_argument_sig
-    elif hasattr(inspect, 'getfullargspec'):
-        has_kw_argument = _has_kw_argument_fullarg
-    elif hasattr(inspect, 'getargspec'):
-        has_kw_argument = _has_kw_argument_arg
-    else:
-        raise RuntimeError('No suitable inspect function found')
-
     if has_kw_argument(v, 'atol'):
         return v
     return lambda answer, result_host, atol: v(answer, result_host)
 
 
+def parse_restrictions(restrictions: list):
+    """" parses restrictions from a list of strings into a callable function """
+
+    # rewrite the restrictions so variables are singled out
+    regex_match_variable = r"([a-zA-Z_$][a-zA-Z_$0-9]*)"
+    suffix = ' and '
+    parsed_restrictions = ""
+    for restriction in restrictions:
+        parsed_restrictions += re.sub(regex_match_variable, r'params["\1"]', restriction) + suffix
+
+    # tidy up the code by removing the last suffix and unecessary spaces
+    parsed_restrictions = parsed_restrictions[:-len(suffix)]
+    parsed_restrictions = parsed_restrictions.strip()
+    parsed_restrictions = " ".join(parsed_restrictions.split())
+
+    # compile into a function
+    parsed_restrictions = f"def restrictions(params): return {parsed_restrictions} \n"
+    code_object = compile(parsed_restrictions, '<string>', 'exec')
+    func = FunctionType(code_object.co_consts[0], globals())
+    return func
+
+
 def process_cache(cache, kernel_options, tuning_options, runner):
     """cache file for storing tuned configurations
 
@@ -580,21 +700,7 @@ def process_cache(cache, kernel_options, tuning_options, runner):
 
     # if file exists
     else:
-        with open(cache, "r") as cachefile:
-            filestr = cachefile.read().strip()
-
-        # if file was not properly closed, pretend it was properly closed
-        if not filestr[-3:] == "}\n}":
-            # remove the trailing comma if any, and append closing brackets
-            if filestr[-1] == ",":
-                filestr = filestr[:-1]
-            filestr = filestr + "}\n}"
-        else:
-            # if it was properly closed, open it for appending new entries
-            with open(cache, "w") as cachefile:
-                cachefile.write(filestr[:-3] + ",")
-
-        cached_data = json.loads(filestr)
+        cached_data = read_cache(cache)
 
         # if in simulation mode, use the device name from the cache file as the runner device name
         if runner.simulation_mode:
@@ -612,6 +718,37 @@ def process_cache(cache, kernel_options, tuning_options, runner):
         tuning_options.cache = cached_data["cache"]
 
 
+def read_cache(cache, open_cache=True):
+    """ Read the cachefile into a dictionary, if open_cache=True prepare the cachefile for appending """
+    with open(cache, "r") as cachefile:
+        filestr = cachefile.read().strip()
+
+    # if file was not properly closed, pretend it was properly closed
+    if not filestr[-3:] == "}\n}":
+        # remove the trailing comma if any, and append closing brackets
+        if filestr[-1] == ",":
+            filestr = filestr[:-1]
+        filestr = filestr + "}\n}"
+    else:
+        if open_cache:
+            # if it was properly closed, open it for appending new entries
+            with open(cache, "w") as cachefile:
+                cachefile.write(filestr[:-3] + ",")
+
+    error_configs = {"InvalidConfig": InvalidConfig(),
+                     "CompilationFailedConfig": CompilationFailedConfig(),
+                     "RuntimeFailedConfig": RuntimeFailedConfig()}
+
+    # replace strings with ErrorConfig instances
+    cache_data = json.loads(filestr)
+    for element in cache_data["cache"].values():
+        for k, v in element.items():
+            if isinstance(v, str) and v in error_configs:
+                element[k] = error_configs[v]
+
+    return cache_data
+
+
 def close_cache(cache):
     if not os.path.isfile(cache):
         raise ValueError("close_cache expects cache file to exist")
@@ -629,23 +766,29 @@ def store_cache(key, params, tuning_options):
     """ stores a new entry (key, params) to the cachefile """
 
     # create converter for dumping numpy objects to JSON
-    def npconverter(obj):
+    def JSONconverter(obj):
         if isinstance(obj, np.integer):
             return int(obj)
-        elif isinstance(obj, np.floating):
+        if isinstance(obj, np.floating):
             return float(obj)
-        elif isinstance(obj, np.ndarray):
+        if isinstance(obj, np.ndarray):
             return obj.tolist()
-        else:
-            return obj.__str__()
+        return obj.__str__()
 
     logging.debug('store_cache called, cache=%s, cachefile=%s' % (tuning_options.cache, tuning_options.cachefile))
     if isinstance(tuning_options.cache, dict):
         if not key in tuning_options.cache:
             tuning_options.cache[key] = params
+
+            # Convert ErrorConfig objects to string, wanted to do this inside the JSONconverter but couldn't get it to work
+            output_params = params.copy()
+            for k,v in output_params.items():
+                if isinstance(v, ErrorConfig):
+                    output_params[k] = str(v)
+
             if tuning_options.cachefile:
                 with open(tuning_options.cachefile, "a") as cachefile:
-                    cachefile.write("\n" + json.dumps({ key: params }, default=npconverter)[1:-1] + ",")
+                    cachefile.write("\n" + json.dumps({ key: output_params }, default=JSONconverter)[1:-1] + ",")
 
 
 def dump_cache(obj: str, tuning_options):
@@ -653,3 +796,46 @@ def dump_cache(obj: str, tuning_options):
     if isinstance(tuning_options.cache, dict) and tuning_options.cachefile:
         with open(tuning_options.cachefile, "a") as cachefile:
             cachefile.write(obj)
+
+
+class MaxProdConstraint(Constraint):
+    """ Constraint enforcing that values of given variables create a product up to a given amount """
+
+    def __init__(self, maxprod):
+        """
+        @param maxprod: Value to be considered as the maximum product
+        @type  maxprod: number
+        @param multipliers: If given, variable values will be multiplied by
+                            the given factors before being prod to be checked
+        """
+        self._maxprod = maxprod
+
+    def preProcess(self, variables, domains, constraints, vconstraints):
+        Constraint.preProcess(self, variables, domains, constraints, vconstraints)
+        maxprod = self._maxprod
+        for variable in variables:
+            domain = domains[variable]
+            for value in domain[:]:
+                if value > maxprod:
+                    domain.remove(value)
+
+    def __call__(self, variables, domains, assignments, forwardcheck=False):
+        maxprod = self._maxprod
+        prod = 1
+        for variable in variables:
+            if variable in assignments:
+                prod *= assignments[variable]
+        if isinstance(prod, float):
+            prod = round(prod, 10)
+        if prod > maxprod:
+            return False
+        if forwardcheck:
+            for variable in variables:
+                if variable not in assignments:
+                    domain = domains[variable]
+                    for value in domain[:]:
+                        if prod * value > maxprod:
+                            domain.hideValue(value)
+                    if not domain:
+                        return False
+        return True
