@@ -15,20 +15,20 @@ def test_get_bounds_x0_eps():
     tune_params['x'] = [0, 1, 2, 3, 4]
 
     tuning_options["tune_params"] = tune_params
+    tuning_options["restrictions"] = None
     tuning_options["strategy_options"] = {}
 
-    bounds, x0, eps = minimize.get_bounds_x0_eps(tuning_options)
+    bounds, x0, eps = minimize.get_bounds_x0_eps(tuning_options, 1024)
 
     assert bounds == [(0.0, 1.0)]
-    assert x0 == [0.5]
+    assert x0 >= 0.0 and x0 <= 1.0
     assert eps == 0.2
 
     tuning_options["scaling"] = False
 
-    bounds, x0, eps = minimize.get_bounds_x0_eps(tuning_options)
+    bounds, x0, eps = minimize.get_bounds_x0_eps(tuning_options, 1024)
 
     assert bounds == [(0, 4)]
-    assert x0 == [2.0]
     assert eps == 1.0
 
 
